import sys
import os
import glob
import pandas as pd
import argparse
import prody


def parse_arguments():
    """
            Parse user arguments

            Output: list with all the user arguments
        """
    # All the docstrings are very provisional and some of them are old, they would be changed in further steps!!
    parser = argparse.ArgumentParser(description="""""")
    required_named = parser.add_argument_group('required named arguments')
    # Growing related arguments
    required_named.add_argument("-tpdb", "--tar_pdb", required=True,
                                help="""Target PDB file.""")
    required_named.add_argument("-rpdb", "--ref_pdb", required=True,
                                help="""Reference PDB file.""")

    parser.add_argument("-ch", "--chain", default="L",
                        help="Ligand chain.")
    parser.add_argument("-d", "--dist", default=4,
                        help="Interaction distance threshold.")
    parser.add_argument("-w2r", "--write2report", default=False, action='store_true',
                        help="Write a report file with the interactions.")
    parser.add_argument("-ah", "--addh", default=False, action='store_true',
                        help="If set it takes into account ligand H's in the interaction.")
    parser.add_argument("-o", "--outfile", default=False,
                        help="If set it takes into account ligand H's in the interaction.")

    args = parser.parse_args()

    return args.tar_pdb, args.ref_pdb, args.chain, args.dist, args.write2report, args.addh, args.outfile


def pdb2prody(pdb_file):
    pdb_in_prody = prody.parsePDB(pdb_file)
    return pdb_in_prody


def get_atoms_of_chain(prody_object, ligand_chain, add_H):
    if not add_H:
        ligand = prody_object.select("heavy and chain {}".format(ligand_chain))
    else:
        ligand = prody_object.select("chain {}".format(ligand_chain))
    return ligand.getNames()


def get_elements_of_chain(prody_object, ligand_chain, add_H):
    if not add_H:
        ligand = prody_object.select("heavy and chain {}".format(ligand_chain))
    else:
        ligand = prody_object.select("chain {}".format(ligand_chain))
    return ligand.getElements()


def get_resname_of_chain(prody_object, ligand_chain):
    ligand = prody_object.select("chain {}".format(ligand_chain))
    return ligand.getResnames()[0]


def get_resnum_of_chain(prody_object, ligand_chain):
    ligand = prody_object.select("chain {}".format(ligand_chain))
    return ligand.getResid()[0]


def get_protein_atoms_at_interaction_distance(atom, pdb_prody, distance=4):
    atoms_close = pdb_prody.select("(protein or water) and within {} of name {}".format(distance, atom))
    if atoms_close == None:
        print("interaction not found")
        return [""]
    else:
        return atoms_close.getNames()


def get_protein_residues_at_interaction_distance(atom, pdb_prody, distance=4):
    atoms_close = pdb_prody.select("(protein or water) and within {} of name {}".format(distance, atom))
    residues = []
    if atoms_close == None:
        print("interaction not found")
        return residues
    else:
        resnums_close = atoms_close.getResnums()
        resnames_close = atoms_close.getResnames()
        for num, resname in zip(resnums_close, resnames_close):
            residues.append("{}{}".format(num, resname))
        return residues


def get_protein_residues_and_atoms_at_interaction_distance(atom, pdb_prody, distance=4):
    atoms_list = get_protein_atoms_at_interaction_distance(atom, pdb_prody, distance)
    residues_list = get_protein_residues_at_interaction_distance(atom, pdb_prody, distance)
    result = []
    for atom_at_dist, residue in zip(atoms_list, residues_list):
        result.append((atom, atom_at_dist, residue))
    return result


def write_report(atom_map, pdb_file):
    lines = []
    for contact in atom_map:
        line = "{:4} {:2} {:4} {}\n".format(contact[0], contact[1], contact[2], contact[3])
        lines.append(line)
    with open("liginteraction_report_{}.txt".format(os.path.splitext(pdb_file)[0]), "w") as report_file:
        report_file.writelines(lines)


def get_interaction_map(pdb_input, ligand_chain, distance, write2report, addh=False):
    pdb = pdb2prody(pdb_input)
    ligand_atoms = get_atoms_of_chain(pdb, ligand_chain, addh)
    elements = get_elements_of_chain(pdb, ligand_chain, addh)
    atom_map = []
    for element, atom in zip(elements, ligand_atoms):
        if not addh:
            if element == "H":
                pass
            else:
                for line in get_protein_residues_and_atoms_at_interaction_distance(atom, pdb, distance):
                    ligatomname, aaatomname, aatype = line
                    print("{:4} {:2} {:4} {}".format(ligatomname, element, aaatomname, aatype))
                    atom_map.append((ligatomname, element, aaatomname, aatype))
        else:
            for line in get_protein_residues_and_atoms_at_interaction_distance(atom, pdb, distance):
                ligatomname, aaatomname, aatype = line
                print("{:4} {:2} {:4} {}".format(ligatomname, element, aaatomname, aatype))
                atom_map.append((ligatomname, element, aaatomname, aatype))
    if write2report:
        write_report(atom_map, pdb_input)
    return atom_map


def get_aminoacids_at_interaction_dist(list_of_interacting_atoms):
    interacting_residues_list = []
    for interacting_atom in list_of_interacting_atoms:
        ligatom = interacting_atom[0]
        ligelement = interacting_atom[1]
        residue = interacting_atom[3]
        interacting_residues_list.append((ligatom, ligelement, residue))
    interacting_residues_list_unique = set(interacting_residues_list)
    return interacting_residues_list_unique


def add_origen_of_differents(list_of_target, list_of_differents):
    new_list = []
    for interaction in list_of_differents:
        if interaction in list_of_target:
            interaction = list(interaction)
            interaction.append("T")
            new_list.append(interaction)
        else:
            interaction = list(interaction)
            interaction.append("R")
            new_list.append(interaction)
    return new_list


def write2report_of_differences(differences_list, output_name):
    output_lines = []
    for interaction in sorted(differences_list):
        line = "{:4} {:2} {:7} {}\n".format(interaction[0], interaction[1], interaction[2], interaction[3])
        output_lines.append(line)
    with open(output_name, "w") as out_file:
        for line in output_lines:
            out_file.writelines(line)


def select_atom_given_name_type_and_num(prody_object, resnum, atom_name):
    selection = prody_object.select("resnum {} and name {}".format(resnum, atom_name))
    return selection


def read_atoms_to_get_distance(filename):
    atoms_to_compute = []
    with open(filename, "r") as instructions:
        lines = instructions.readlines()
        for line in lines:
            atom_name_ligand = line[0]
            atom_name_aa = line[1]
            resnum = line[2]
            resname = line[3]
            atoms_to_compute.append((atom_name_ligand, atom_name_aa, resnum, resname))
    return atoms_to_compute


def compute_distances_given_instruction(pdb_object, instruction, chain = "L"):
    ligand_resname = get_resname_of_chain(pdb_object, chain)
    ligand_resnum = get_resnum_of_chain(pdb_object, chain)
    atom_name_ligand, atom_name_aa, resnum_aa, resname_aa = instruction
    atom_of_ligand = select_atom_given_name_type_and_num(pdb_object, ligand_resnum, atom_name_ligand)
    atom_of_protein = select_atom_given_name_type_and_num(pdb_object, ligand_resnum, atom_name_ligand)
    distance = prody.calcDistance(atom_of_ligand, atom_of_protein)
    return atom_name_ligand, atom_name_aa, resnum_aa, resname_aa, distance


def main(pdb_target, pdb_reference, ligand_chain, distance, write2report, distances_file = False, addh=False, output_name=False):
    target_interactions = set(get_interaction_map(pdb_target, ligand_chain, distance, write2report, addh))
    reference_intertactions = set(get_interaction_map(pdb_reference, ligand_chain, distance, write2report, addh))
    differences = list(target_interactions - reference_intertactions)
    amount_t_interactions = len(target_interactions)
    amount_r_interactions = len(reference_intertactions)
    print("DIFFERENT ATOMS CONTACTS")
    differences = sorted(differences)
    differences = add_origen_of_differents(target_interactions, differences)
    differences_in_num = len(differences)
    for diff in differences:
        print("{:4} {:2} {:4} {:7} {}".format(diff[0], diff[1], diff[2], diff[3], diff[4]))
    print("Initial atoms at interaction distance: {}\nFinal atoms at interaction distance: {}\nDifferences: {}".format(
                                                                                  amount_r_interactions,
                                                                                  amount_t_interactions,
                                                                                  differences_in_num))
    target_aa_close = get_aminoacids_at_interaction_dist(target_interactions)
    ref_aa_close = get_aminoacids_at_interaction_dist(reference_intertactions)
    differences_aa = list(set(target_aa_close) - set(ref_aa_close))
    differences_aa = add_origen_of_differents(target_aa_close, differences_aa)
    print("DIFFERENT AA CONTACTS")
    for interaction in sorted(differences_aa):
        print("{:4} {:2} {:7} {}".format(interaction[0], interaction[1], interaction[2], interaction[3]))
    amount_aa_t_interactions = len(target_aa_close)
    amount_aa_r_interactions = len(ref_aa_close)
    differences_aa_in_num = len(differences_aa)
    print("Initial amino acids at interaction distance: {}\nFinal amino acids at interaction distance: {}\nDifferences: {}".format(
        amount_aa_r_interactions,
        amount_aa_t_interactions,
        differences_aa_in_num))
    if distances_file:
        for instruction in read_atoms_to_get_distance(distances_file):
            compute_distances_given_instruction(pdb_target, instruction, chain)
    if output_name:
        write2report_of_differences(differences_aa, output_name)


if __name__ == '__main__':
    pdb_target, pdb_reference, chain, dist, write2report, addh, outfile = parse_arguments()
    main(pdb_target, pdb_reference, chain, dist, write2report, addh, outfile)



"""
    Recreate the trajectory fragments to the led to the discovery of a snapshot,
    specified by the tuple (epoch, trajectory, snapshot) and write as a pdb file
"""
from __future__ import print_function
import os
import sys
import argparse
import glob
import itertools
import re
from ast import literal_eval
from AdaptivePELE.utilities import utilities
try:
    basestring
except NameError:
    basestring = str


def parseArguments():
    """
        Parse the command-line options

        :returns: str, str, str, str --  path to file to backtrack,
            path to the result files,
            output path where to write the files, name of the files
    """
    desc = "Write the information related to the conformation network to file\n"
    parser = argparse.ArgumentParser(description=desc)
    parser.add_argument("file_to_backtrack", type=str, help="File of the selected_results folder that you want to"
                                                            "backtrack.")

    parser.add_argument("-p", type=str, default="growing_results", help="Path to the folder which contains the growing"
                                                                        "results of all growing steps.")
    parser.add_argument("-o", type=str, default=None, help="Output path where to write the files")
    parser.add_argument("--name", type=str, default="pathway.pdb", help="Name of the pdb to write the files")
    args = parser.parse_args()
    return args.file_to_backtrack, args.p, args.o, args.name


def main(file_to_backtrack, results_path, outputPath, out_filename):
    """

    :param file_to_backtrack: File of the selected_results folder that you want to backtrack.
    :type file_to_backtrack: str
    :param results_path: Path where the growing simulations are stored.
    :type results_path: str
    :param outputPath: Output folder path.
    :type outputPath: str
    :param out_filename: Output filename prefix.
    :type out_filename: str
    :return: None
    """
    if outputPath is not None:
        outputPath = os.path.join(outputPath, "")
        if not os.path.exists(outputPath):
            os.makedirs(outputPath)
    else:
        outputPath = ""
    if os.path.exists(outputPath+out_filename):
        # If the specified name exists, append a number to distinguish the files
        name, ext = os.path.splitext(out_filename)
        out_filename = "".join([name, "_%d", ext])
        i = 1
        while os.path.exists(outputPath+out_filename % i):
            i += 1
        out_filename %= i
    pathway = []
    sys.stderr.write("Creating pathway...\n")
    # Get information of the input structure
    trajectory, snapshot, growing_id = extract_info_from_selected_file(path_to_selected_file=file_to_backtrack)
    # Obtain trajectory from sampling folder
    sampling_file_from = "sampling_result_{}/trajectory_{}.pdb".format(growing_id, trajectory)
    sampling_models = utilities.getSnapshots(sampling_file_from)
    # Get all snapshots from the input snapshot and add it to pathway
    snapshots = sampling_models[1:snapshot + 1]
    pathway.insert(0, snapshots)
    growing_epochs = glob.glob(os.path.join(results_path, "{}_growing_output*".format(growing_id)))
    if not growing_epochs:
        raise ValueError("Trajectory %s not found!" % os.path.join(results_path))
    for n in range(len(growing_epochs)-1, -1, -1):
        # Reading mapping information
        procMapping = open(os.path.join(results_path, "{}_growing_output{}".format(growing_id, n),
                                        "processorMapping.txt")).read().split(":")
        # Extract the spawning structure of the previous simulation
        cluster_from = procMapping[trajectory-1].rstrip("\n")
        cluster_from_tup = literal_eval(cluster_from)
        # Update the trajectory and snapshot to be used
        trajectory = cluster_from_tup[1]
        snapshot = cluster_from_tup[2]
        # Extract the filename and the snapshot that has spawn
        filename = os.path.join(results_path, "{}_growing_output{}/trajectory_{}.pdb".format(growing_id, n, trajectory))
        print(n, filename, trajectory, snapshot, procMapping)
        snapshots = utilities.getSnapshots(filename)
        if n == 0:
            initial = 0
        else:
            initial = 1
        # Take all MODELS from the snapshot
        snapshots = snapshots[initial:snapshot+1]
        pathway.insert(0, snapshots)

    sys.stderr.write("Writing pathway...\n")
    with open(outputPath+out_filename, "a") as f:
        f.write("ENDMDL\n".join(itertools.chain.from_iterable(pathway)))


def extract_info_from_selected_file(path_to_selected_file):
    pattern_trajectory = re.compile('trajectory_[\d+]\.')
    pattern_snapshot = re.compile('\.[\d+]_')
    trajectory_and_others = re.findall(pattern_trajectory, path_to_selected_file)
    trajectory = re.findall('\d+', trajectory_and_others[0])
    snapshot_and_others = re.findall(pattern_snapshot, path_to_selected_file)
    snapshot = re.findall('\d+', snapshot_and_others[0])
    growing_id = path_to_selected_file.split("epochsampling_result_")[-1].split("_trajectory")[0]
    return int(trajectory[0]), int(snapshot[0]), growing_id


if __name__ == "__main__":
    traj, results_path, output_path, output_filename = parseArguments()
    main(traj, results_path, output_path, output_filename)



import sys
import re
import os
import string
import pandas as pd
import logging

# Getting the name of the module for the log system
logger = logging.getLogger(__name__)


def trajectory_selector(output, path_to_file="/growing_output", report="report", trajectory="trajectory.pdb",
                        criteria="Binding Energy"):
    """
    This function select the step of a trajectory of PELE
    with the minimum value of the criteria selected
    and extract it as a single pdb file.
    """
    # Storing the trajectory as a string
    with open(os.path.join(path_to_file, trajectory), 'r') as input_file:
        file_content = input_file.read()

    # Storing the report file as pandas data-frame
    data = pd.read_csv(os.path.join(path_to_file, report), sep='    ', engine='python')

    # Now, select only the columns correspondent to the numberOfAcceptedPeleSteps
    # (if you sum 1 to this number you can obtain the step in the trajectory) and the criteria
    selected_data = data.loc[1:, ['numberOfAcceptedPeleSteps', criteria]]

    # Find the minimum value of the criteria
    min_energy = selected_data.min(axis=0)[1]

    # Once we have this value, find the step that correspond to this minimum value
    min_trajectory = selected_data.loc[selected_data[criteria] == min_energy, 'numberOfAcceptedPeleSteps'].iloc[0]

    # Using this step number we will select the MODEL (step of the trajectory) that correspond to this value.
    # We do min_trajectory+1 because the difference between accepted steps and models numbers
    trajectory_selected = re.search('MODEL\s+%d(.*?)ENDMDL' %int(min_trajectory+1), file_content, re.DOTALL)

    # Writing this step into a PDB with a single structure. We will do it with a list in order to reduce
    # computing time
    output_file_content = []
    output_file_content.append("MODEL     {}".format(int(min_trajectory+1)))
    output_file_content.append("{}".format(trajectory_selected.group(1)))
    output_file_content.append("ENDMDL")
    output_file_content = "\n".join(output_file_content)
    with open(output, 'w') as output_file:
        output_file.write(output_file_content)
        logger.info(":::.MODEL     {} has been selected.:::".format(int(min_trajectory + 1)))


# Old function. We are not using it now.
def change_ligandname(input_file, output):
    """
    From an input pdb file this function replace the first character of the ligand name
    string to the next one in alphabetic order
    """
    # Creating a list of capital letters
    letters = list(string.ascii_uppercase)
    with open(output, 'w') as output_f:
        with open(input_file) as input_f:
            for line in input_f:
                # We only want to read lines that contain information about the ligand
                if line.startswith("HETATM"):
                    # This is the ligandname of the original file, column 4.
                    ligandname_old = line.split()[3]

                    # We will transform this name into a list of letters (strings in python are unmutable)
                    ligandname_new = list(ligandname_old)

                    # Now, the index that we will select is the first letter of the ligandname
                    index = letters.index(ligandname_old[0])

                    # Doing this we are going to transform the first letter of the ligand name into
                    # the next in the alphabet. F.ex: CMB -> DMB
                    new_letter = letters[index+1]
                    ligandname_new[0] = new_letter

                    # Join again all the letters into a string
                    ligandname_new = "".join(ligandname_new)

                    # Write it in the file
                    output_f.write(line.replace(ligandname_old, ligandname_new))
                else:
                    # The rest of the file that don't have information about the ligand will stay unchanged
                    output_f.write(line)
    # We would like to delete the temporary input files
    os.remove(input_file)



import sys
import os
from string import Template


class TemplateBuilder(object):
    """
        Description: Base clase to substitute de keywords
        int the already templetize file.

        e.g.

        If we have a file like that --> dededea $RESIDUENAME $CHAIN
        to replace $RESIDUENAME to ASL and $CHAIN to Z we should do:

        file= /path/to/file
        keywords= { "RESIDUENAME" : "ASL", "CHAIN": "Z" }

        TemplateBuilder(file, keywords)

    """

    def __init__(self, file, keywords):

        self.file = file
        self.keywords = keywords
        self.fill_in()

    def fill_in(self):
        """
        Fill the control file in
        """
        with open(os.path.join(self.file), 'r') as infile:
            confile_data = infile.read()

        confile_template = Template(confile_data) 

        confile_text = confile_template.safe_substitute(self.keywords)

        with open(os.path.join(self.file), 'w') as outfile:
            outfile.write(confile_text)


import argparse
import os
import sys
import itertools
import lib_prep.pdb_modifier as pm
from AdaptivePELE.utilities import utilities


def parse_arguments():
    """
        Parse the command-line options

        :returns: str, str, str --  path to file to backtrack,
            output path where to write the files, name of the files
    """
    desc = "Adds the connectivity information to a trajectory file that does not have it.\n"
    parser = argparse.ArgumentParser(description=desc)
    parser.add_argument("pathway", type=str, help="Trajectory file with the backtracked pathway.")
    parser.add_argument("pdb_with_connects", type=str, help="PDB of the final structure prepared. Must contain"
                                                            " CONNECTS section. Remember to not delete water molecules!")
    parser.add_argument("--ligand_chain", type=str, default="L", help="Chain of the ligand.")
    parser.add_argument("-o", type=str, default=None, help="Output file with the new trajectory.")

    args = parser.parse_args()
    return args.pathway, args.pdb_with_connects, args.ligand_chain, args.o


def main(pathway, pdb_with_connects, ligand_chain="L", outfilename=None):
    pdb = pm.PDB(pdb_with_connects, chain=ligand_chain)
    ligand = "".join(pdb.get_atoms_of_chain())
    connects = "".join(pdb.read_conect())
    ligand_with_connects = ligand + connects
    # Temporary file to extract the indexes of the ligand
    with open("ligand_with_connects_tmp.pdb", "w") as out:
        out.write(ligand_with_connects)
    pdb = pm.PDB(in_pdb="ligand_with_connects_tmp.pdb", chain=ligand_chain)
    names_dictionary = pdb.get_names_dictionary()
    os.remove("ligand_with_connects_tmp.pdb")
    pathway_snapshots = utilities.getSnapshots(pathway)
    new_pathway = []
    sys.stderr.write("Rebuilding indexes...\n")
    for snap in pathway_snapshots:
        with open("snap_tmp.pdb", "w") as out:
            out.write(snap)
        pdb = pm.PDB(in_pdb="snap_tmp.pdb", chain=ligand_chain)
        os.remove("snap_tmp.pdb")
        ligand = pdb.get_atoms_of_chain()
        complex_no_ligand = sorted(list(set(pdb.read_atoms_section()) ^ set(ligand)))
        new_ligand = []
        for index, name in names_dictionary.items():
            for line in ligand:
                pdb_name = pm.get_atom_pdb_name_from_line(line).strip()
                if name == pdb_name:
                    new_line = pm.set_index_to_line(line, index)
                    new_ligand.append(new_line)
        new_ligand_with_connects = "".join(new_ligand) + connects
        new_complex = "".join(complex_no_ligand) + new_ligand_with_connects
        new_pathway.append(new_complex)

    sys.stderr.write("Writing connects...\n")
    if not outfilename:
        outfilename = "{}_connected.pdb".format(pathway.split(".pdb")[0])
    if os.path.exists(outfilename):
        os.remove(outfilename)
    for n, model in enumerate(new_pathway):
        with open(outfilename, "a") as output:
            output.write("MODEL {}\n".format(n))
            output.write(model)
            output.write("ENDMDL\n")


if __name__ == "__main__":
    pathway, pdb_with_connects, chain, out = parse_arguments()
    main(pathway, pdb_with_connects, chain, out)




import sys
import os
import re
import glob
import pandas as pd
import argparse


def parse_arguments():
    """
            Parse user arguments

            Output: list with all the user arguments
        """
    # All the docstrings are very provisional and some of them are old, they would be changed in further steps!!
    parser = argparse.ArgumentParser(description="""Computes the mean of the 25% lowest values of the sampling simulation
    for each fragment grown. """)
    required_named = parser.add_argument_group('required named arguments')
    # Growing related arguments
    required_named.add_argument("path_to_analyze",
                                help="""Path of the folder to be analyzed, where all simulations are stored.""")

    parser.add_argument("-r", "--rep_pref", default='report_',
                                help="""Prefix of the report file: usually 'report_'. """)
    parser.add_argument("-s", "--steps", default=False,
                        help="Used to filter by N PELE steps")
    parser.add_argument("-f", "--file", default=False,
                        help="Used to set the flag 'export file? to True'")
    parser.add_argument("-o", "--out", default="",
                        help="Name of the output file")
    parser.add_argument("-e", "--equil_folder", default="sampling_results*",
                        help="Prefix of the equilibration folder (where the results are)")
    parser.add_argument("-c", "--col", default="Binding Energy",
                        help="Name of the column that we will use to find the minimum value of the PELE report")
    parser.add_argument("-q", "--quart", default=0.25,
                        help="Quartile that will be used to compute the mean")
    parser.add_argument("-csv", "--export_csv", default=True,
                        help="By default a csv file with a summary of all reports will be stored. If you do not"
                        "want to save it, set the flag to False.")
    parser.add_argument("-l", "--limit_column", default=None, type=str,
                        help="Column used to subset the results.")
    parser.add_argument("-lu", "--limit_up", default=None, type=float,
                        help="Top value to create the new subset of samples.")
    parser.add_argument("-ld", "--limit_down", default=None, type=float,
                        help="Lowest value to create the new subset of samples.")

    args = parser.parse_args()

    return args.path_to_analyze, args.rep_pref, args.steps, args.file, args.out, args.equil_folder, args.col, \
           args.quart, args.export_csv, args.limit_column, args.limit_up, args.limit_down


def pele_report2pandas(path, export=True):
    """
        This function merge the content of different report for PELE simulations in a single file pandas Data Frame.
    """
    data = []
    report_list = glob.glob('{}*'.format(path))
    for report in report_list:
        tmp_data = pd.read_csv(report, sep='    ', engine='python')
        tmp_data = tmp_data.iloc[1:]  # We must discard the first row
        processor = re.findall('\d+$'.format(path), report)
        tmp_data['Processor'] = processor[0]
        data.append(tmp_data)
    result = pd.concat(data)
    if export:
        path_to_folder = "/".join(path.split("/")[0:-1])
        new_dir = path_to_folder + "/summary"
        if not os.path.exists(new_dir):
            os.mkdir(new_dir)
        while True:
            counter = 1
            try:
                result.to_csv("{}/summary_report_{}.csv".format(new_dir, counter))
                print("Data saved in {}/summary_report_{}.csv".format(new_dir, counter))
                break
            except FileExistsError:
                counter =+ 1
                result.to_csv("{}/summary_report_{}.csv".format(new_dir, counter))

    return result


def select_subset_by_steps(dataframe, steps):
    """
    Given a pandas dataframe from PELE's result it returns a subset of n PELE steps.
    :param dataframe: pandas object
    :param steps: int
    :return: dataframe
    """
    subset = dataframe[dataframe['Step'] <= int(steps)]
    return subset


def get_min_value(dataframe, column):
    minimum = dataframe.loc[dataframe[column] == dataframe[column].min()]
    if len(minimum) > 1:
        return minimum.iloc[0][column].min()
    else:
        return minimum[column].min()


def compute_mean_quantile(dataframe, column, quantile_value, limit_col=None, limit_up=None, limit_down=None):
    if limit_col:
        if not limit_up:
            raise ValueError("You must fill the argument '--limit_up'!")
        if not limit_down:
            raise ValueError("You must fill the argument '--limit_down'!")
        dataframe = dataframe[dataframe[limit_col] > float(limit_down)]
        dataframe = dataframe[dataframe[limit_col] < float(limit_up)]
    dataframe = dataframe[dataframe[column] < dataframe[column].quantile(quantile_value)]
    dataframe = dataframe[column]
    mean_subset = dataframe.mean()
    return mean_subset


def get_csv(dataframe, output_path):
    dataframe.to_csv(path_or_buf=output_path, sep='\t', header=True, index=False)


def compute_sterr(dataframe, column, quantile_value):
    dataframe = dataframe[dataframe[column] < dataframe[column].quantile(quantile_value)]
    dataframe = dataframe[column]
    err_subset = dataframe.sem()
    return err_subset


def get_score_for_folder(report_prefix, path_to_equilibration, steps=False,
                         column="Binding Energy", quantile_value=0.25, export=True,
                         limit_col=None, limit_up=None, limit_down=None):
    df = pele_report2pandas(os.path.join(path_to_equilibration, report_prefix), export)
    if steps:
        df = select_subset_by_steps(df, steps)
    mean_quartile = compute_mean_quantile(df, column, quantile_value, limit_col, limit_up, limit_down)
    results = [path_to_equilibration, mean_quartile]
    print("SCORING results: {}	{}".format(results[0], results[1]))
    return results


def analyse_at_epoch(report_prefix, path_to_equilibration, execution_dir, steps=False, column="Binding Energy", quantile_value=0.25):
    result = get_score_for_folder(report_prefix=report_prefix, path_to_equilibration=path_to_equilibration,
                                  steps=steps, column=column, quantile_value=quantile_value)
    out_file = os.path.join(execution_dir, "simulation_score_summary.tsv")
    if os.path.exists(out_file):
        #Pandas new version compatibility
        try:
            df = pd.read_csv(out_file, sep="\t", header=0, ignore_index=True)
        except TypeError:
            df = pd.read_csv(out_file, sep="\t", header=0, index_col=False)
        df = pd.concat([df, pd.DataFrame([result],  columns=["Fragment_Results_Folder", "Score"])])
    else:
        df = pd.DataFrame([result], columns=["Fragment_Results_Folder", "Score"])
    df.to_csv(out_file, sep="\t", index=False)


def main(report_prefix, path_to_equilibration, equil_pattern="equilibration*", steps=False, out_report=False,
         column="Binding Energy", quantile_value=0.25, export=True, limit_col=None, limit_up=None, limit_down=None):
    folder_list = glob.glob(os.path.join(path_to_equilibration, equil_pattern))
    for folder in list(folder_list):
        try:
            result = get_score_for_folder(report_prefix=report_prefix, path_to_equilibration=folder, steps=steps,
                                          column=column, quantile_value=quantile_value, export=export,
                                          limit_col=limit_col, limit_up=limit_up, limit_down=limit_down)
            line_of_report = "{}\t{:.2f}\n".format(result[0], float(result[1]))
            print(line_of_report)
            if out_report:
                if os.path.exists(out_report):
                    with open(out_report, "a") as report:
                        report.write(line_of_report)
                else:
                    with open(out_report, "w") as report:
                        report.write("# FragmentResultsFolder\tFrAG-score\n")
        except Exception as e:
            print("ERROR {} in {}".format(e, folder))


if __name__ == '__main__':
    path, rep_pref, steps, file, out, equil_folder, col, quart, export, limit_col, limit_up, limit_down= \
        parse_arguments()
    main(report_prefix=rep_pref, path_to_equilibration=path, equil_pattern=equil_folder, steps=steps, out_report=out,
         column=col, quantile_value=quart, export=export, limit_col=limit_col, limit_up=limit_up, limit_down=limit_down)






import os
import logging
# Local import
import frag_pele.constants as c


# Getting the name of the module for the log system
logger = logging.getLogger(__name__)


def check_and_create_folder(path):
    if not os.path.exists(path):
        os.mkdir(path)


def check_and_create_DataLocal(working_dir):
    # Get current path
    check_and_create_folder((os.path.join(working_dir, "DataLocal")))
    check_and_create_folder((os.path.join(working_dir, "DataLocal/LigandRotamerLibs")))
    check_and_create_folder((os.path.join(working_dir, "DataLocal/Templates")))
    check_and_create_folder((os.path.join(working_dir, "DataLocal/Templates/OPLS2005")))
    check_and_create_folder((os.path.join(working_dir, "DataLocal/Templates/OPLS2005/HeteroAtoms")))
    check_and_create_folder((os.path.join(working_dir, "DataLocal/Templates/OPLS2005/HeteroAtoms", "templates_generated")))


def check_and_create_results_folder(results_folder, workdir):
    check_and_create_folder(os.path.join(workdir, c.OUTPUT_FOLDER))
    check_and_create_folder(results_folder)


def check_and_create_pdb_clusters_folder(pdbout_folder, iteration):
    if iteration == 0:
        check_and_create_folder(pdbout_folder)
        check_and_create_folder(os.path.join(pdbout_folder, "{}".format(iteration)))
    else:
        check_and_create_folder(os.path.join(pdbout_folder, "{}".format(iteration)))



import matplotlib.pyplot as plt
import numpy as np
import argparse


def parse_arguments():
    """
            Parse user arguments

            Output: list with all the user arguments
        """
    # All the docstrings are very provisional and some of them are old, they would be changed in further steps!!
    parser = argparse.ArgumentParser(description="""Script to perform plots. You can choose between different
    type of plots (depending on your input file): 'boxplot_single' if your input is the result of the analysis
    between two structures, for which you have computed the RMSD and the CA distance between them; 'boxplot_differences'
    if you want to show the comparison of RMSD and CA distance in two different files; and 'boxplot_atom_distances'
    if you have calculated atom-atom distances in three structures in order to show the differences.""")
    required_named = parser.add_argument_group('required named arguments')
    # Growing related arguments
    required_named.add_argument("-t", "--type", required=True, choices=['boxplot_single','boxplot_differences',
                                                                        'boxplot_atom_distances'],
                                help="""Plot type that you want to do. Choose between: boxplot_differences, 
                                boxplot_single. """)
    required_named.add_argument("-i", "--input", required=True,
                                help="""Path of the input file.""")

    parser.add_argument("-i2", "--input_2", default=False,
                        help=""""Path of the input file 2, only set if you want to do comparisons.""")

    parser.add_argument("-i3", "--input_3", default=False,
                        help=""""Path of the input file 2, only set if you want to do comparisons.""")

    args = parser.parse_args()

    return args.type, args.input, args.input_2, args.input_3


def plot_aminoacid_single(report_file1):

    parser.add_argument("-t", "--title", default="",
                        help=""""Title of the graph.""")
    parser.add_argument("-ts", "--title_size", default=14,
                        help=""""Fontsize of the title.""")
    parser.add_argument("-x", "--x_label", default="",
                        help=""""Label to name the X-axis.""")
    parser.add_argument("-y", "--y_label", default="",
                        help=""""Label to name the Y-axis.""")
    parser.add_argument("-xs", "--x_size", default="",
                        help=""""Fontsize of the label to name the X-axis.""")
    parser.add_argument("-ys", "--y_size", default="",
                        help=""""Fontsize of the label to name the Y-axis.""")
    parser.add_argument("-l1", "--legend1", default="",
                        help=""""Name to put in the legend for the group 1.""")
    parser.add_argument("-l2", "--legend2", default="",
                        help=""""Name to put in the legend for the group 2.""")
    parser.add_argument("-l3", "--legend3", default="",
                        help=""""Name to put in the legend for the group 3.""")
    parser.add_argument("-l4", "--legend4", default="",
                        help=""""Name to put in the legend for the group 4.""")
    parser.add_argument("-xts", "--xtick_size", default="",
                        help=""""Fontsize of the label to name the grups of the X-axis.""")

    args = parser.parse_args()

    return args.type, args.input, args.input_2, args.input_3, args.title, args.title_size, args.x_label, args.y_label, \
           args.x_size, args.y_size, args.legend1, args.legend2, args.legend3, args.legend4, args.xtick_size


def plot_aminoacid_single(report_file1, x_label, y_label, label_legend_1, label_legend_2, title, x_label_fontsize = 12,
                          y_label_fontsize = 12, title_fontsize = 14, xticklabels_fontsize = 10):
    """
    Given a single report file the function create a boxplot with two bars for n groups.
    :param report_file1: report file path
    :return: boxplot
    """
    # Reading report file 1
    with open(report_file1) as results:
        lines_file1 = results.readlines()
    n_groups_1 = len(lines_file1)
    residue_and_type_1 = []
    total_rmsd_1 = []
    distances_cas_1 = []
    for line in lines_file1:
        resid, type, rmsd_total, distance_ca, diff = line.split("\t")
        residue_and_type_1.append("{}{}".format(resid, type))
        total_rmsd_1.append(float(rmsd_total))
        distances_cas_1.append(float(distance_ca))

    fig, ax = plt.subplots()

    index = np.arange(n_groups_1)
    bar_width = 0.35

    opacity = 0.5

    rects1 = ax.bar(index, total_rmsd_1, bar_width,
                    alpha=opacity, color='r',
                    label='RMSD')

    rects2 = ax.bar(index + bar_width, distances_cas_1, bar_width,
                    alpha=opacity, color='b',
                    label=r'C$\alpha$ distance')

    ax.set_xlabel('Residues', fontsize=12)
    ax.set_ylabel(r'$\AA$',fontsize=12)
    ax.set_title('Comparison of sidechains between Initial and Final structures', fontsize=14)
    ax.set_xticks(index + bar_width / 2)
    ax.set_xticklabels(residue_and_type_1, fontsize=10)
                    label='{}'.format(label_legend_1))

    rects2 = ax.bar(index + bar_width, distances_cas_1, bar_width,
                    alpha=opacity, color='b',
                    label='{}'.format(label_legend_2))

    ax.set_xlabel('{}'.format(x_label), fontsize=x_label_fontsize)
    ax.set_ylabel('{}'.format(y_label), fontsize=y_label_fontsize)
    ax.set_title('{}'.format(title), fontsize=title_fontsize)
    ax.set_xticks(index + bar_width / 2)
    ax.set_xticklabels(residue_and_type_1, fontsize=xticklabels_fontsize)
    ax.legend()

    fig.tight_layout()
    plt.show()


def plot_aminoacid_differences(report_file1, report_file2, x_label, y_label, label_legend_1, label_legend_2,
                               label_legend_3, label_legend_4, title, x_label_fontsize = 12, y_label_fontsize = 12,
                               title_fontsize = 14, xticklabels_fontsize = 10):
    """
    Given two report files the function create a boxplot with four bars for n groups (two bars for each file in order
    to compare them.).
    :param report_file1: report file path
    :param report_file2: report file path. This report requires the same labels and metrics than in the first one.
    :return: boxplot
    """
    # Reading report file 1
    with open(report_file1) as results:
        lines_file1 = results.readlines()
    n_groups_1 = len(lines_file1)
    residue_and_type_1 = []
    total_rmsd_1 = []
    distances_cas_1 = []
    for line in lines_file1:
        resid, type, rmsd_total, distance_ca, diff = line.split("\t")
        residue_and_type_1.append("{}{}".format(resid, type))
        total_rmsd_1.append(float(rmsd_total))
        distances_cas_1.append(float(distance_ca))

    # Reading report file 2
    with open(report_file2) as results:
        lines_file2 = results.readlines()
    n_groups_2 = len(lines_file2)
    residue_and_type_2 = []

    total_rmsd_2 = []
    distances_cas_2 = []

    for line in lines_file2:
        resid, type, rmsd_total, distance_ca, diff = line.split("\t")
        residue_and_type_2.append("{}{}".format(resid, type))
        total_rmsd_2.append(float(rmsd_total))
        distances_cas_2.append(float(distance_ca))

    # Simple check to ensure equal groups
    if not n_groups_1 == n_groups_2:
        exit("Different group lengths")
    if not residue_and_type_1 == residue_and_type_2:
        exit("The residues differ from the file 1 and file 2")

    fig, ax = plt.subplots()

    index = np.arange(n_groups_1)
    bar_width = 0.20

    opacity = 0.7

    rects1 = ax.bar(index, total_rmsd_1, bar_width,
                    alpha=opacity, color='b',
                    label='total RMSD initial')

    rects2 = ax.bar(index + bar_width, distances_cas_1, bar_width,
                    alpha=opacity, color='g',
                    label=r'C$\alpha$ distance initial')
    rects3 = ax.bar(index + bar_width*2, total_rmsd_2, bar_width,
                    alpha=opacity, color='r',
                    label='total RMSD final')

    rects4 = ax.bar(index + bar_width*3, distances_cas_2, bar_width,
                    alpha=opacity, color='y',
                    label=r'C$\alpha$ distance final')

    ax.set_xlabel('Residues', fontsize=12)
    ax.set_ylabel(r'$\AA$',fontsize=12)
    ax.set_title(r'RMSDs and C$\alpha$ distances for amino acid', fontsize=14)
    ax.set_xticks(index + bar_width / 4)
    ax.set_xticklabels(residue_and_type_1, fontsize=10)
                    label='{}'.format(label_legend_1))

    rects2 = ax.bar(index + bar_width, distances_cas_1, bar_width,
                    alpha=opacity, color='g',
                    label='{}'.format(label_legend_2))
    rects3 = ax.bar(index + bar_width*2, total_rmsd_2, bar_width,
                    alpha=opacity, color='r',
                    label='{}'.format(label_legend_3))

    rects4 = ax.bar(index + bar_width*3, distances_cas_2, bar_width,
                    alpha=opacity, color='y',
                    label='{}'.format(label_legend_4))

    ax.set_xlabel('{}'.format(x_label), fontsize=x_label_fontsize)
    ax.set_ylabel('{}'.format(y_label),fontsize=y_label_fontsize)
    ax.set_title('{}'.format(title), fontsize=title_fontsize)
    ax.set_xticks(index + bar_width / 4)
    ax.set_xticklabels(residue_and_type_1, fontsize=xticklabels_fontsize)
    ax.legend()

    fig.tight_layout()
    plt.show()

def plot_atom_distances(input_initial, input_final, input_cristal, x_label, y_label, label_legend_1, label_legend_2,
                               label_legend_3, title, x_label_fontsize = 12, y_label_fontsize = 12,
                               title_fontsize = 14, xticklabels_fontsize = 10):
    """
    Given three report files, the function create a boxplot with three bars for n groups (one bar for each file in order
    to compare them.).
    :param input_initial: report file (with distances) path of the initial structure
    :param input_final: report file (with distances) path of the final structure (model)
    :param input_cristal: report file (with distances) path of the crystal
    :return: boxplot
    """
    # Reading report file 1
    with open(input_initial) as results:
        lines_file1 = results.readlines()
    n_groups_initial = len(lines_file1)
    atoms_initial = []
    distances_initial = []
    for line in lines_file1:
        resid, atoms1, atoms2, distance = line.split()
        atoms_initial.append("{}-{}{}".format(atoms1, resid, atoms2))
        distances_initial.append(float(distance))

    # Reading report file 2
    with open(input_final) as results:
        lines_file2 = results.readlines()

    n_groups_final = len(lines_file2)
    atoms_final = []
    distances_final = []
    for line in lines_file2:
        resid, atoms1, atoms2, distance = line.split()
        atoms_final.append("{}-{}{}".format(atoms1, resid, atoms2))
        distances_final.append(float(distance))

    # Reading report file 3
    with open(input_cristal) as results:
        lines_file3 = results.readlines()

    n_groups_cristal = len(lines_file3)
    atoms_cristal = []
    distances_cristal = []
    for line in lines_file3:
        resid, atoms1, atoms2, distance = line.split()
        atoms_cristal.append("{}-{}{}".format(atoms1, resid, atoms2))
        distances_cristal.append(float(distance))
    # Simple check to ensure equal groups

    if not n_groups_initial == n_groups_final == n_groups_cristal:
        print(n_groups_initial, n_groups_final, n_groups_cristal)
        exit("Different group lengths")
    if not atoms_initial == atoms_final == atoms_cristal:
        exit("The atoms differ between files")

    fig, ax = plt.subplots()

    index = np.arange(n_groups_initial)
    bar_width = 0.05
    opacity = 0.5

    rects1 = ax.bar(index, distances_initial, bar_width,
                    alpha=opacity, color='r',
                    label='Initial')

    rects2 = ax.bar(index + bar_width, distances_final, bar_width,
                    alpha=opacity, color='b',
                    label='Model')
    rects3 = ax.bar(index + bar_width*2, distances_cristal, bar_width,
                    alpha=opacity, color='g',
                    label='Crystal')

    ax.set_xlabel('Interaction', fontsize=12)
    ax.set_ylabel(r'Distance ($\AA$)', fontsize=12)
    ax.set_title('Comparison of distances in EGFR', fontsize=14)
    ax.set_xticks(index + bar_width)
    ax.set_xticklabels(atoms_initial, fontsize=10)
                    label='{}'.format(label_legend_1))

    rects2 = ax.bar(index + bar_width + 0.02, distances_final, bar_width,
                    alpha=opacity, color='b',
                    label='{}'.format(label_legend_2))
    rects3 = ax.bar(index + bar_width*2 + 0.04, distances_cristal, bar_width,
                    alpha=opacity, color='g',
                    label='{}'.format(label_legend_3))

    ax.set_xlabel('{}'.format(x_label), fontsize=x_label_fontsize)
    ax.set_ylabel('{}'.format(y_label), fontsize=y_label_fontsize)
    ax.set_title('{}'.format(title), fontsize=title_fontsize)
    ax.set_xticks(index + bar_width + 0.02)
    ax.set_xticklabels(atoms_initial, fontsize=xticklabels_fontsize)
    ax.legend()

    fig.tight_layout()
    plt.show()


def plot_comparison_between_r(input, x_label, y_label, label_legend_1, label_legend_2,
                               label_legend_3, title, x_label_fontsize = 12, y_label_fontsize = 12,
                               title_fontsize = 14, xticklabels_fontsize = 10):

    fig, ax = plt.subplots()

    index = np.arange(n_groups_initial)
    bar_width = 0.05

    opacity = 0.5

    rects1 = ax.bar(index, distances_initial, bar_width,
                    alpha=opacity, color='r',
                    label='{}'.format(label_legend_1))

    rects2 = ax.bar(index + bar_width + 0.02, distances_final, bar_width,
                    alpha=opacity, color='b',
                    label='{}'.format(label_legend_2))
    rects3 = ax.bar(index + bar_width * 2 + 0.04, distances_cristal, bar_width,
                    alpha=opacity, color='g',
                    label='{}'.format(label_legend_3))

    ax.set_xlabel('{}'.format(x_label), fontsize=x_label_fontsize)
    ax.set_ylabel('{}'.format(y_label), fontsize=y_label_fontsize)
    ax.set_title('{}'.format(title), fontsize=title_fontsize)
    ax.set_xticks(index + bar_width + 0.02)
    ax.set_xticklabels(atoms_initial, fontsize=xticklabels_fontsize)
    ax.legend()

    fig.tight_layout()
    plt.show()


if __name__ == '__main__':
    type, input, input_2, input_3, title, title_size, x_label, y_label, x_size, y_size, legend1, legend2, legend3, \
    legend4, xtick_size = parse_arguments()
    if type == "boxplot_single":
        plot_aminoacid_single(input, x_label, y_label, legend1, legend2, title,  x_size, y_size, title_size, xtick_size)
    if type == "boxplot_differences":
        plot_aminoacid_differences(input, input_2,  x_label, y_label, legend1, legend2, legend3, legend4, title,
                                   x_size, y_size, title_size, xtick_size)
    if type == "boxplot_atom_distances":
        plot_atom_distances(input, input_2, input_3, x_label, y_label, legend1, legend2, legend3,
                            title, x_size, y_size, title_size, xtick_size)



"""
$Revision: 2.0.0 $
For ligands:
Reads in a maestro mae file and makes a "rotamer enabled" template and the rotamer library to accompany it.  
This consists of finding the backbone core that results in the least number of child bonds rotated with any rotatable bond rotation. 
Reads in the rotatable bonds from a macromodel atomtyping (should be easily modifyiable to read them from stdin).  
Hetgrp_ffgen is used for atomtyping and to determine the geometry in the template fromt the mae.  The mae must only have the residue to atomtype in it. 

Builds rotamer libraries for arbitrary ligand molecules by building and combining rotamer libraries.  There are two general algorithms that are implemented.  This first is using macromodel (or some other external tool) to sample the degrees of freedom and converting the resulting ensemble into a rotamer library.  The second is identifying the rotatable bonds, again using macromodel, and assigning rotamer libraries to these bonds.  For most bonds a simple freely rotatable library ( 0,10,20,30...360), but in the case of closed rings special libraries are built using macromodel sampling.  These component rotamer libraries are then arranged into groups for use in PLOP.  Each group consists of a single tree rooted at the central core.  This core can either be used chosen, or will be chosen based on an algorithm that minimizes the number of bond lengths from the farthest leeaf to the trunk.  Any built rotamer libraries are placed in the current directory and a file called <resname>.rot.assign is also written.  This tells PLOP how to assemble the full combinatoric library that will be used in sidehchain prediction/monte carlo.     

For unnatural amino acids:
Requires: 1) a maestro mae file of an unnatural amino acid with no NMA or ACE termini; the N-H and C=0 are to be left as they are found in a peptide
Outputs: 1) a re-ordered template file (old one is backed up in FILENMAE.hetgrp_ffgen)
         2) a PLOP nonstandard residue specification for pasting into a PLOP control file, both to stdout and to <maefile>_torsions.txt


Options:
   
   --core            Set a core atom 
   --mtor            Maximum number of torsion per group
   --n               Maximum number of entries in the rotamer file
   --ext_charges     Import charges from file e.g. (Inclusion of quantum charges)
   --clean           Clean Intermiadiate files

  
Mae file should be properly atomtyped and a residue name is needed

Most common problem:  As part of this procedure the pdb atom names are often renamed when they are not unique. Alsothis procedure works best if the ligand is minimized.  For these two reasons an atomtyped, minimzed version of the input ligand is written to (input).PlopRotTemp.pdb.  If at all possible, use the ligand structure and atom names from this file in any subsequent plop runs.    

examples:
Build a rotamer library for the following ligand
$SCHRODINGER/utilities/python PlopRotTemp.py 3ert_lig.mae


Make libraries for rotatable bonds in ligand.mae up to a maximum of 4 rotatable bonds in each library
$SCHRODINGER/utilities/python PlopRotTemp.py ligand.mae --mtor=4

For a given ligand named LIG the following files will be created:
ligz               - Template file for use in PLOP, its zmatrix matches the libraries created
LIG.rot.assign     - Summary of all libraries build or used for this ligand read into plop with the command
                     "rot assign all"

---------------------------------------------------------------------------------------------------------



All jobs run on the localhost


USAGE: "$SCHRODINGER/utilities/python main.py [file.mae] --options valueoption"

HELP: $SCHRODINGER/utilities/python main.py --help

"""


import argparse
import sys
import os
import re
import shutil
import PlopRotTemp as pl
from template.templateBuilder import TemplateBuilder



def main(mae_file, max_tors=-1, nrot=1000, user_core_atom=-1, mae_charges = False, clean = False, out_temp=".",
         out_rot=".", gridres="10.0"):

    #Defaults
    nrot = nrot
    max_tors = max_tors
    user_core_atom = user_core_atom
    template_file = ""
    debug = 0  # 1 means don't run exteral commands (assumes output is already there)
    conf_file = 'none';
    output_template_file = ""
    gridres = gridres
    nsamp = 10000
    Ecut = 100
    use_rings = 0
    do_init_min = 0
    max_dist_eq = 0.25
    user_tors = []
    back_tors = []
    back_algorithm = "none"
    back_conf_file = ""
    hetgrp_opt = ""
    OPLS = "2005"
    user_fixed_bonds = []
    files2clean = []
    use_mult_lib = 1
    run_conf = 0
    algorithm = "MCMM"
    gridres_oh = gridres
    unnat_res = 0  # for old-style PLOP nonstandard side chain
    resno = 1  #for old-style PLOP nonstandard side chain
    chain = 'A'  #for old-style PLOP nonstandard side chain
    grow = 0
    tree = 0  # for old-style PLOP nonstandard ligand tree-style torsion reordering
    R_group_root_atom_name = 'None'  # which atom do you want to start sampling at?

    if (unnat_res == 1):
        init_min = 0  #the input mae file is for a peptide and will not have a suitable Lewis structure
        if (template_file == ""):
            print("Cannot use unnatural residues without pre-made template files!")
            sys.exit(-1)
        use_mult_lib = 1  # so a dummy conformational search is performed just to see which bonds are rotatble
        use_rings = 0  #for now; I'm not sure that ring torsions will follow the tree pattern appropriately, this would need testing
        #For now, just try low energy ring conformations


    atomnames = pl.find_names_in_mae(mae_file)
    
    pl.check_repite_names(atomnames)


    root = pl.get_root_path(mae_file)

    print("\n")
    print("INPUT")
    print("mae_file {}".format(mae_file))
    print("root {}".format(root))
    print("OPLS {}".format(OPLS))
    print("hetgrp options '{}'".format(hetgrp_opt))
    print("User template file '{}'".format(template_file))
    print("User output template file '{}'".format(output_template_file))
    print("\n")


    #Build a template file

    print("TEMPLATE GENERATION")

    resname=pl.find_resnames_in_mae(mae_file)
    template_output = os.path.join(out_temp, "{}z".format(resname[0].lower()))

    template_builder = TemplateBuilder(mae_file, template_output)

    [template_file, output_template_file, mae_file_hetgrp_ffgen, files, resname] = \
        template_builder.build_template(mae_charges)

    print(output_template_file)



    

    print("\n")
    if (do_init_min == 1):
        mcu_mini = mu.ComUtil(ffld='opls2005', serial=True, solv=True, nant=False, demx=True)
        mcu_mini.SOLV[2] = 1  # water
        mini_root = root + "_mini"
        com_file = mcu_mini.mini(mae_file_hetgrp_ffgen, mini_root + '.com')
        print('\nMINIMIZATION\nRunning minimization: {0} -> {1} -out.mae\n'.format(mae_file_hetgrp_ffgen, mini_root))
        if (not debug):
            cmd = mcu_mini.getLaunchCommand(com_file)
            job = jc.launch_job(cmd)
            job.wait()
            files2clean.append(mini_root + '-out.mae')
            files2clean.append(mini_root + '.log')
    #       files2clean.append(mini_root + '-out.tmp')
    #        files2clean.append(mini_root + '.com')
        mae_min_file = mini_root + "-out.mae"
    else:
        print('\nSkipping Minimization\n ')
        mae_min_file = mae_file_hetgrp_ffgen


    print("\n")

    if (unnat_res == 1):
        [mae_num, parent, rank, tors, use_rings, group, tors_ring_num] = \
            pl.FindCoreAA(mae_min_file, user_fixed_bonds, use_rings, resname, use_mult_lib, user_core_atom, user_tors)
        tors_ring_num = []
        for t in tors: tors_ring_num.append(0);
    else:
        print('FINDING CORE')
        if (grow == 1 and user_core_atom == -1): user_core_atom = -2
        #######Assign_rank--> Extremely slow!
        [mae_num, parent, rank, tors, use_rings, group, back_tors, tors_ring_num] = \
            pl.FindCore(mae_min_file, user_fixed_bonds, use_rings, resname, \
                     use_mult_lib, user_core_atom, user_tors, back_tors, max_tors, R_group_root_atom_name)
    
    if (use_rings == 1):
        print("Found flexible rings")

    newtors = []
    if (unnat_res == 1 or grow == 1 ):
        newtors = pl.ReorderTorsionsAA(tors, mae_num)


    #Change from mae files atom numbering to the template file ones
    #Convert Torsions to match the template file atom numbering
    #Ring numbers don't have to be changed
    [mae2temp, temp2mae] = pl.MatchTempMaeAtoms(mae_min_file, template_file)

    old_atom_num = [];
    new_tors = [];
    new_back_tors = [];
    for i in mae_num:
        old_atom_num.append(-100)
    for i in range(len(mae2temp)):
        old_atom_num[i] = mae2temp[mae_num[i]]
    for i in range(len(tors)):
        temp = [mae2temp[tors[i][0]], mae2temp[tors[i][1]]]
        new_tors.append(temp)
    for i in range(len(back_tors)):
        temp = [mae2temp[back_tors[i][0]], mae2temp[back_tors[i][1]]]
        new_back_tors.append(temp)
    tors = []
    for i in range(len(new_tors)):
        temp = [old_atom_num.index(new_tors[i][0]), old_atom_num.index(new_tors[i][1])]
        tors.append(temp)
    back_tors = []
    for i in range(len(new_back_tors)):
        temp = [old_atom_num.index(new_back_tors[i][0]), old_atom_num.index(new_back_tors[i][1])]
        back_tors.append(temp)


    #Make (or read) original tempalte file
    print('\n')
    print('CREATE ROTAMER TEMPLATE FILE: {}'.format(output_template_file))
    names = pl.ReorderTemplate(old_atom_num, parent, rank, template_file, output_template_file, mae_file, 
        R_group_root_atom_name=R_group_root_atom_name)

    [tors, tors_ring_num, zmat_atoms] = pl.FindTorsAtom(tors, tors_ring_num, parent)
    #Eliminate Torsions in the backbone (included when entire rings are included in the torsions)
    [tors, tors_ring_num, zmat_atoms] = pl.EliminateBackboneTors(tors, tors_ring_num, zmat_atoms, rank)

    pl.replace_vdwr_from_library(output_template_file)

    if (unnat_res == 1 or grow == 1):
        mynonstandard = pl.TetherRotBonds(mae_file, chain, resno, log_file, newtors)
        mynonstandard.output_rotbonds(R_group_root_atom_name=R_group_root_atom_name)


    else:

        # Reorder the torsions
        for i in range(len(tors)):
            tors[i].sort()
        for i in range(len(tors)):
            for j in range(i + 1, len(tors)):
                if (tors[i] > tors[j]):
                    temp = tors[i];
                    tors[i] = tors[j];
                    tors[j] = temp;
                    temp = tors_ring_num[i];
                    tors_ring_num[i] = tors_ring_num[j];
                    tors_ring_num[j] = temp

        [tors, tors_ring_num, zmat_atoms] = pl.FindTorsAtom(tors, tors_ring_num, parent)


    ################################CHANGE MACROMODEL CONFORMATIONAL SEARCH######################
    #Run the conformational Search
    conf_root = root + "_conf"
    if (conf_file == conf_root + '-out.mae'):
        raise Exception('Must use different name for conformational file')

    if (conf_file == ''):
        run_conf = 1
    else:
        run_conf = 0

    back_lib = "";
    if (unnat_res != 1):  
        if (conf_file != 'none'):
            rotamers_file = pl.make_libraries(resname, conf_file, root, names, zmat_atoms, group, use_rings, use_mult_lib,
                           output_template_file, gridres, debug, out_rot)
            print("\n")
            print("CREATE ROTAMER LIBRARY")
            print(rotamers_file)
            print("\n")


        
        else:
            if (len(zmat_atoms) > 0):
                ring_libs = pl.build_ring_libs(mae_min_file, root, resname, tors, \
                                            tors_ring_num, names, rank, parent, old_atom_num, mae2temp, gridres,
                                            files2clean, debug)
     
            else:
                ring_libs = []
                print("No rotatable sidechains found")
            
            rotamers_file = pl.find_build_lib(resname, mae_min_file, root, tors, names, group, gridres, gridres_oh, use_rings, back_lib,
                                  tors_ring_num, ring_libs, debug, out_rot)
            print("\n")
            print("CREATE ROTAMER LIBRARY")
            print(rotamers_file)
            print("\n")




    if (clean):
        for file in files2clean:
            print('Removing Intermediate File: {}'.format(file))
            os.remove(file)

    return output_template_file, rotamers_file



def parse_args():

    parser = argparse.ArgumentParser()
    parser.add_argument("mae_file", type=str, help="ligand maestro mae file")
    parser.add_argument("--core", type=int, help="Give one atom of the core section", default=-1)
    parser.add_argument("--mtor", type=int, help="Gives the maximum number of torsions allowed in each group.  Will freeze bonds to extend the core if necessary.", default=-1)
    parser.add_argument("--n", type=int, help="Maximum Number of Entries in Rotamer File", default=1000)
    parser.add_argument("--mae_charges", help="Use charges in mae", action='store_true')
    parser.add_argument("--clean", help="Whether to clean up all the intermediate files", action='store_true')
    parser.add_argument("--gridres", help="Rotamer resolution", type=str, default="10.0")
    parser.add_argument("--out_temp", type=str, help="Template output path")
    parser.add_argument("--out_rot", type=str, help="Rotamer ouput path")
    args = parser.parse_args()

    
    return args.mae_file, args.mtor, args.n, args.core, args.mae_charges, args.clean, args.out_temp, args.out_rot, args.gridres

if __name__ == "__main__":
    mae_file, mtor, n, core, mae_charge, clean, out_temp, out_rot, gridres = parse_args() 
    template, rotamers_file = main(mae_file, mtor, n, core, mae_charge, clean, out_temp, out_rot, gridres=gridres)
    

    print("########################################################################")
    print("\n{} template and {} rotamer library has been successfully created in {}\n").format(
        template,rotamers_file, os.getcwd())
    print("########################################################################")


    




# General imports
import os
import shutil
import subprocess
import logging

# Local imports
import frag_pele.Helpers.templatize as tp

# Getting the name of the module for the log system
logger = logging.getLogger(__name__)


def control_file_modifier(control_template, pdb, license, working_dir, overlap=0.7, step=0,
                          results_path="/growing_output", steps=6, chain="L", constraints=" ", center="",
                          temperature=1000, seed=1279183, steering=0, translation_high=0.05, translation_low=0.02,
                          rotation_high=0.10, rotation_low=0.05, radius=4):
    """
    This function creates n control files for each intermediate template created in order to change
    the logPath, reportPath and trajectoryPath to have all control files prepared for PELE simulations.
    """

    ctrl_fold_name = os.path.join(working_dir, "control_folder")
    controlfile_path = control_template
    control_template = os.path.basename(control_template)

    # Then, in the main loop we will do a copy of control files, so we will print this in the logger
    logger.info("Intermediate control files created will be stored in '{}'".format(ctrl_fold_name))

    # As pdb is a list of complexes, we have to create one line per complex in pdb and then put them all in the control
    list_of_lines_complex = []
    for complex in pdb:
        control_file_complex = '{"files" : [{"path": "%s" }] }' % (complex)
        list_of_lines_complex.append(control_file_complex)
    lines_complex = ",\n".join(list_of_lines_complex)

    # Definition of the keywords that we are going to substitute from the template
    keywords = {"LICENSE": license,
                "RESULTS_PATH": results_path,
                "CHAIN": chain,
                "CONSTRAINTS": constraints,
                "CENTER": center, 
                "PDB": lines_complex,
                "STEPS": steps,
                "OVERLAP": overlap,
                "TEMPERATURE": temperature,
                "SEED": seed,
                "STEERING": steering,
                "TRANSLATION_HIGH": translation_high,
                "TRANSLATION_LOW": translation_low,
                "ROTATION_HIGH": rotation_high,
                "ROTATION_LOW": rotation_low,
                "RADIUS": radius
                }

    # Creation of a folder where we are going to contain our control files, just if needed
    if not os.path.exists(ctrl_fold_name):
        os.mkdir(ctrl_fold_name)

    # Create a copy of the control template in the control folder, because templatize.py replace the original template
    if not os.path.exists(os.path.join(ctrl_fold_name, control_template)):
        shutil.copyfile(controlfile_path, os.path.join(ctrl_fold_name, control_template))
        shutil.copyfile(os.path.join(ctrl_fold_name, control_template), os.path.join(working_dir, control_template))

    # Else, if has been created this means that we already have a template in this folder, so we will need a copy of the
    # file in the main folder to then replace the template for a real control file
    else:
        shutil.copyfile(os.path.join(ctrl_fold_name, control_template), os.path.join(working_dir, control_template))

    # Modifying the control file template
    tp.TemplateBuilder(os.path.join(working_dir, control_template), keywords)
    # Make a copy in the control files folder
    simulation_file = os.path.join(ctrl_fold_name, "{}_{}".format(step, control_template))
    shutil.copyfile(os.path.join(working_dir, control_template), simulation_file)
    logger.info("{}_{} has been created successfully!".format(step, control_template))

    return simulation_file


def simulation_runner(path_to_pele, control_in, cpus=4):
    """
    Runs a PELE simulation with the parameters described in the input control file.

    Input:

    path_to_pele --> Complete path to PELE folder

    control_in --> Name of the control file with the parameters to run PELE
    """
    if cpus:
        cpus = int(cpus)
        if cpus < 2:
            logger.critical("Sorry, to run mpi PELE you need at least 2 CPUs!")
        else:
            logger.info("Starting PELE simulation. You will run mpi PELE with {} cores.".format(cpus))
            cmd = "mpirun -np {} {} {}".format(cpus, path_to_pele, control_in)
            print(cmd)
            logger.info("Running {}".format(cmd))
            subprocess.call(cmd.split())
    else:
        logger.info("Starting PELE simulation. You will run serial PELE.")
        cmd = "{} {}".format(path_to_pele, control_in)
        logger.info("Running {}".format(cmd))
        subprocess.call(cmd.split())



import sys
import os
import re
import mdtraj
import glob
import argparse
import pandas as pd
import matplotlib.pyplot as plt


def parse_arguments():
    """
            Parse user arguments

            Output: list with all the user arguments
        """
    # All the docstrings are very provisional and some of them are old, they would be changed in further steps!!
    parser = argparse.ArgumentParser(description="""""")
    required_named = parser.add_argument_group('required named arguments')
    # Growing related arguments
    required_named.add_argument("-pdbr", "--pdb_reference",required=True,
                                help="""Reference pdb file.""")
    required_named.add_argument("-p", "--path", required=True,
                                help="""Path to folder to be analyzed. """)

    parser.add_argument("-r", "--resname", default="GRW",
                        help="Resame of the ligand.")
    parser.add_argument("-c", "--csv", default="summary/summary_report_1.csv",
                        help="Relative path to csv file")


    # Plot arguments
    parser.add_argument("-y", "--ycol", default="RMSD",
                        help="Column name of the report file that you want to use as X axis in the plot.")
    parser.add_argument("-op", "--plot", default=False, action="store_true",
                        help="Flag to plot the results.")

    args = parser.parse_args()

    return args.pdb_reference, args.path, args.resname, args.csv, args.ycol, args.plot


def check_common_residues(pdb_target_prody, pdb_reference_prody):
    list_of_types_and_resnums_target= []
    list_of_types_and_resnums_reference = []
    for resnum, resname in zip(pdb_target_prody.getResnums(), pdb_target_prody.getResnames()):
        list_of_types_and_resnums_target.append((resnum, resname))
    for resnum, resname in zip(pdb_reference_prody.getResnums(), pdb_reference_prody.getResnames()):
        list_of_types_and_resnums_reference.append((resnum, resname))
    common_residues = sorted(list(set(list_of_types_and_resnums_target).intersection(list_of_types_and_resnums_reference)))
    only_resnums = []
    for residue in common_residues:
        resnum = residue[0]
        only_resnums.append(resnum)
    return only_resnums


def ligands_rmsd_calculator(pdb_target, pdb_reference, resname="GRW"):
    """

    :param pdb_target: problem pdb file
    :param pdb_reference: reference pdb file
    :param write2report: if True export results in a file
    :param write2pdb: pdb file with the result of the superposition between "pdb_target" and "pdb_reference"
    :param ligand_chain: name of the chain of the ligand
    :return: superpose the backbone of the pdb_target to the pdb_reference and computes the RMSD of the ligand
    """
    # Reparation of PDB files if it is needed
    repair_pdbs(pdb_target)
    # Load the data to mdtraj
    target = mdtraj.load(pdb_target)
    reference = mdtraj.load(pdb_reference)
    # Get the indexes of heavy atoms of the ligands
    ligand = reference.topology.select('resname "{}"'.format(resname))
    rmsd = mdtraj.rmsd(target, reference=reference, atom_indices=ligand)
    return rmsd


def repair_pdbs(pdb_file):
    with open(pdb_file) as pdb:
        pdb_content = pdb.readlines()
    pdb_new = []
    for line in pdb_content:
        atom_name = line[12:16]
        if line.startswith("HETATM") and atom_name.strip().startswith("G"):
            line_new = list(line)
            size = len(atom_name.strip())
            white_spaces = 3-size
            line_new[12:16] = line[77:78] + atom_name.strip() + " "*white_spaces
            modified = "".join(line_new)
            pdb_new.append(modified)
        else:
            pdb_new.append(line)
    pdb_modified = "".join(pdb_new)
    with open(pdb_file, "w") as write_output:
        write_output.write(pdb_modified)


def sort_atoms_by_name(pdb_file):
    with open(pdb_file) as pdb:
        pdb_content = pdb.readlines()
    pdb_content_sorted = (sorted(pdb_content, key=lambda atom_name: atom_name[12:16]))
    new_pdb_content = "".join(pdb_content_sorted)
    with open(pdb_file, "w") as write_output:
        write_output.write(new_pdb_content)


def compute_rmsd_in_serie(pdb_reference, path, resname="GRW", pattern_to_csv="summary/summary_report_1.csv", plot=False,
                          y=None):
    equilibration_files = sorted(glob.glob("{}/trajectory_[0-9]*.pdb".format(path)))
    csv = os.path.join(path, pattern_to_csv)
    df = pd.read_csv(csv)
    for file in equilibration_files:
        result = ligands_rmsd_calculator(file, pdb_reference, resname=resname)
        processor = re.findall(r'\d+', file.split("/")[-1])[0]
        try:
            indexes = search_indexes_with_certain_value_in_column(df, int(processor), "Processor")
            for rmsd, index in zip(result, indexes):
                add_data_to_dataframe(dataframe=df, value_to_add=rmsd, row_to_add=index, column_name="rmsd")
        except Exception as e:
            print(e)
    # Overwriting the initial report
    df.to_csv(csv)
    if plot:
        filename = os.path.join(path, "{}_rmsd.png".format(y))
        plot_results(df, xcolumn="rmsd", ycolumn=y, outfile=filename)


def add_data_to_dataframe(dataframe, value_to_add, row_to_add, column_name="new_column"):
    dataframe.loc[row_to_add, column_name] = value_to_add


def search_indexes_with_certain_value_in_column(dataframe, value_to_seach, column_to_seach):
    idx = dataframe.index[dataframe[column_to_seach] == value_to_seach]
    return idx


def check_if_multimodels_and_split(pdb_file):
    with open(pdb_file) as pdb:
        content = pdb.readlines()
    pdbs = []
    lines_in_model = []
    for line in content:
        lines_in_model.append(line)
        if "ENDMDL" in line:
            pdbs.append("".join(lines_in_model))
            lines_in_model = []
    return pdbs


def get_report_file_from_trajectory(trajectory_path, report_prefix):
    regex = re.compile(r'\d+')
    sufix = regex.findall(trajectory_path)[-1]
    prefix = trajectory_path.split("/")[0:-1]
    new_prefix = "/".join(prefix)
    path_completed = os.path.join(new_prefix, report_prefix+sufix)
    return path_completed


def plot_results(dataframe, xcolumn, ycolumn, outfile):
    dataframe.plot(xcolumn, ycolumn, kind="scatter")
    plt.xlabel(xcolumn)
    plt.ylabel(ycolumn)
    plt.savefig(outfile)
    plt.show()


def get_snapshot(trajectory_file, model, output_file):
    pdb = check_if_multimodels_and_split(trajectory_file)[model]
    with open(output_file,"w") as pdb2write:
        pdb2write.write(pdb)


def get_min_row(data, column_number):
    minimum = data.loc[data[column_number] == data[column_number].min()]
    return minimum


def get_pdb_by_min_value(report_file, column_number):
    data = pd.read_csv(report_file, sep="\t", header=None)
    minium_row = get_min_row(data, column_number)
    return minium_row[0].values[0], minium_row[1].values[0]


if __name__ == '__main__':
    pdb_reference, path, resname, csv, ycol, plot = parse_arguments()
    compute_rmsd_in_serie(pdb_reference, path, pattern_to_csv=csv, resname=resname, plot=plot, y=ycol)




import numpy
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path
import frag_pele as fp

try:
    from Cython.Build import cythonize
    from Cython.Distutils import build_ext
except ImportError:
    use_cython = False
else:
    use_cython = True

here = path.abspath(path.dirname(__file__))
ext_modules = []
cmdclass = {}

# Get the long description from the README file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()
setup(
    name="frag_pele",
    version=fp.__version__,
    description='FragPELE, a new tool for in silico hit-to-lead drug design, capable of growing a frag_pelement into a core while exploring the protein-ligand conformational space',
    long_description=long_description,
    url="https://github.com/carlesperez94/frag_pele/",
    author='Carles Perez Lopez, Daniel Soler Viladrich',
    author_email='daniel.soler@nostrumbiodiscovery.com, carlesperez94@gmail.com',
    license='',
    packages=find_packages(exclude=['docs', 'tests']),
    include_package_data=True,
    include_dirs=[numpy.get_include()],
    install_requires=['cython', 'numpy',  'scipy', 'matplotlib', 'biopython ', 'pandas',  'prody==1.10', 'pytest',
    'AdaptivePELE', 'lib_prep'],
    cmdclass=cmdclass,
    ext_modules=ext_modules,  # accepts a glob pattern
    #include_dirs=[numpy.get_include()],
    classifiers=(
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 2.7",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX :: Linux",
        "Intended Audience :: Science/Research"
    ),
    project_urls={
    'Documentation': 'https://carlesperez94.github.io/frag_pele/',
    'Source': 'https://carlesperez94.github.io/frag_pele/',
'Tracker': 'https://github.com/carlesperez94/frag_pele/issues',
},
)




import re


class Helper(object):

	def __init__(self, *args, **kwargs):
		pass
		
	def preproces_file_lines(self, file):
	  with open(file, "r") as f:
	    lines = f.readlines()
	    for i, line in enumerate(lines):
	      line = re.sub(' +',' ',line)
	      line = line.strip('\n').strip()
	      lines[i] = line
	    return lines


import sys
import mdtraj as md
from AdaptivePELE.clustering import clustering, thresholdcalculator
from AdaptivePELE.spawning import spawning, densitycalculator
from AdaptivePELE.constants import constants
from AdaptivePELE.utilities import utilities
import pandas as pd
import glob
import os


def cluster_traject(resname, trajToDistribute, columnToChoose, distance_contact, clusterThreshold, path_to_cluster,
                    output_path, mapping_out, epsilon=0.5, report_basename="report", condition="min",
                    metricweights="linear", nclusters=5):

    outputPathConst = constants.OutputPathConstants(output_path)
    outputPathConst.tmpFolder = output_path
    outputPathConst.buildTmpFolderConstants(outputPathConst.tmpFolder)
    utilities.makeFolder(outputPathConst.tmpFolder)

    thresholdCalc = thresholdcalculator.ThresholdCalculatorConstant(value=clusterThreshold)
    similarityEval = clustering.CMSimilarityEvaluator("Jaccard")
    clusteringObject = clustering.ContactMapAccumulativeClustering(thresholdCalc, similarityEval, resname=resname,
                                                                   reportBaseFilename=report_basename,
                                                                   columnOfReportFile=columnToChoose,
                                                                   contactThresholdDistance=distance_contact,
                                                                   altSelection=True)

    clusteringObject.cluster([path_to_cluster], ignoreFirstRow=True)
    spawning_params = spawning.SpawningParams()
    spawning_params.reportFilename = report_basename
    spawning_params.epsilon = epsilon
    spawning_params.nclusters = nclusters
    spawning_params.metricWeights = metricweights
    spawning_params.condition = condition

    density = densitycalculator.NullDensityCalculator()
    spawningObject = spawning.EpsilonDegeneracyCalculator(spawning_params, density)
    degeneracy = spawningObject.calculate(clusteringObject.clusters, trajToDistribute, spawning_params)
    spawningObject.log()

    _, procMapping = spawningObject.writeSpawningInitialStructures(outputPathConst, degeneracy, clusteringObject,
                                                                   0)
    processorManagerFilename = "processorMapping.txt"
    utilities.writeProcessorMappingToDisk(mapping_out, processorManagerFilename, procMapping)


def get_column_num(path, header_column, report_basename="report"):
    reports = glob.glob(os.path.join(path, "*{}*".format(report_basename)))
    try:
        reports[0]
    except IndexError:
        raise IndexError("Not report file found. Check you are in adaptive's or Pele root folder")

    data = pd.read_csv(reports[0], sep='    ', engine='python')
    header_list = data.columns.values.tolist()
    column_number = header_list.index(header_column)
    return column_number


def check_atom_overlapping(pdb_list, ligand_resname="GRW"):
    pdb_wrong = []
    for pdb in pdb_list:
        structure = md.load(pdb)
        ligand = structure.topology.select("resname {}".format(ligand_resname))
        clash = md.compute_neighbors(structure, cutoff=0.002, query_indices=ligand)
        if len(clash[0]) != 0:
            pdb_wrong.append(pdb)
    return pdb_wrong



import prody
import logging
import glob
import multiprocessing as mp

# Getting the name of the module for the log system
logger = logging.getLogger(__name__)


class Detector:
    def __init__(self, pdb, threshold, dihedrals, lig_chain="L"):
        self.pdb = pdb
        self.threshold = threshold
        self.lig_chain = lig_chain
        self.dihedrals = dihedrals

    def read_pdb(self):
        pdb = prody.parsePDB(self.pdb)
        return pdb

    def get_ligand(self):
        pdb = self.read_pdb()
        ligand = pdb.select("chain {}".format(self.lig_chain))
        if ligand is None:
            logger.critical("Wrong chain selected!")
        elif ligand.ishetero:
            return ligand
        else:
            logger.critical("The selected chain does not contain heteroatoms!")

    def select_atoms(self):
        ligand = self.get_ligand()
        dihedral_list = []
        for dihedral in self.dihedrals:
            atoms = ligand.select("name {} {} {} {}".format(dihedral[0], dihedral[1], dihedral[2], dihedral[3]))
            new_list = []
            for atom in dihedral:
                for atom2 in atoms:
                    if atom == atom2.getName():
                        new_list.append(atom2)
                        break
            dihedral_list.append(new_list)
        return dihedral_list

    def read_dihedral(self):

        dihedral_list = self.select_atoms()
        dihedral_results = []
        for dihedral in dihedral_list:
            atoms = []
            for atom in dihedral:
                atoms.append(atom)
            dihedral_angle = prody.calcDihedral(atoms[0], atoms[1], atoms[2], atoms[3])
            dihedral_results.append(dihedral_angle)
        return dihedral_results

    def check_threshold_dihedral(self):
        value_list = self.read_dihedral()
        check = True
        for value in value_list:
            if abs(self.threshold) > value:
                check = False
        return check


def checker(pdb, threshold, dihedrals, lig_chain):
    new_detector = Detector(pdb, threshold, dihedrals, lig_chain)
    return new_detector.check_threshold_dihedral(), pdb


def check_folder(folder, threshold, dihedrals, lig_chain="L", processors=4):
    dict_of_checks = {}
    list_of_pdbs = glob.glob("{}/*.pdb".format(folder))
    multi = []
    pool = mp.Pool(processors)
    for pdb in list_of_pdbs:
        multi.append(pool.apply_async(checker, [pdb, threshold, dihedrals, lig_chain]))
    for process in multi:
        flag, pdb = process.get()
        dict_of_checks[pdb] = flag
    return dict_of_checks






